/************************************************************************/
/*																		*/
/*	System		: 78K0/IB2 Fluorecent Ballast Evaluation Board			*/
/*				  (EZ-BLST-003)											*/
/*																		*/
/*	Function	: Dimming process										*/
/*																		*/
/*	CPU			: uPD78F0756											*/
/*																		*/
/*	Last Update	: 2009/08/10(MON)										*/
/*																		*/
/*	File name	: dimming.c												*/
/*																		*/
/************************************************************************/
#pragma	SFR

/*========================================================================
	Include files
========================================================================*/
#include "defination.h"

/*========================================================================
	Function definition
========================================================================*/
void fn_ini_ad_variables(void);			/* Initialize the variables for A/D conversion */
void fn_ini_status_variables(void);		/* Initialize the lamp status variables */
void fn_start_ad(void);					/* Start A/D conversion */
void fn_accumulate_ad(void);			/* Accumulate A/D conversion result */
void fn_average_ad(void);				/* Average A/D conversion result */
void fn_check_request(void);			/* Check dimming request */
void fn_detect_fault(void);				/* Detect fault for emergency stop */
void fn_control_vdc(void);				/* Control PFC output */
void fn_control_dim(void);				/* Control dimming operation */
void fn_dim_off(void);					/* Off the lamp */
void fn_dim_ignition(void);				/* Ignite the lamp */
void fn_dim_min(void);					/* Turn down to minimum level */
void fn_dim_max(void);					/* Turn up to maximum level */

/*========================================================================
	Variable definition
========================================================================*/
/*----- for inverter control -----*/
sreg unsigned short ushInvCycle;	/* Inverter cycle to rewrite TX1CR3 */

/*----- for dimming process -----*/
sreg unsigned char ucDimMode;		/* Dimming mode
										0: Lamp off
										1: Lamp ignition
										2: Maximum lighting
										3: Minimum lighting */
sreg unsigned char ucDimState;		/* Dimming state
										0: Lamp off
										1: Boost DC output voltage
										2: Preheat lamp
										3: Sweep inverter frequency for lamp ignition
										4: Keep lighting to wait stable lighting
										5: Stable lighting */
sreg unsigned char ucDimCnt;		/* Dimming time counter for lighting process */

/*----- for A/D conversion -----*/
/* "NUM_TOTAL_ADCH" defines the total number of A/D conversion channel (4ch) */
sreg unsigned short ushAdSum[NUM_TOTAL_ADCH];	/* Accumulated A/D value */
sreg unsigned char ucAdCnt[NUM_TOTAL_ADCH];		/* A/D value addition counter */
sreg unsigned char ucAdAve[NUM_TOTAL_ADCH];		/* A/D average value */
sreg unsigned char ucVdcOutOld;					/* A/D value of previous DC output voltage */
sreg unsigned char ucAdCh;						/* A/D channel pointer */
sreg unsigned char ucVac;						/* AC supply voltage : ucVac*10[V] */

/*----- status flags -----*/
boolean bLampIgnition;		/* Lamp ignition flag
								0 : befor ignition
								1 : after ignition
							*/
boolean bVdcAutoControl;	/* VDC output auto control flag
								0 : no auto control
								1 : under auto control
							*/
boolean bDimmingReq;		/* Dimming request flag
								0 : no dimming request
								1 : under dimming request
							*/

/*========================================================================
	Constant value definition
========================================================================*/
/*----- A/D conversion channel table -----*/
const unsigned char ucAni[NUM_TOTAL_ADCH] = {
	ADCH_VDCOUT,	/* [0] : DC output voltage */
	ADCH_LAMP,		/* [1] : Lamp detection */
	ADCH_VACIN,		/* [2] : AC supply voltage */
	ADCH_VOLUME		/* [3] : Volume input */
};

/*----- On-time of PFC output table for preheating -----*/
const unsigned short ushPfcPreheat[NUM_STEP_VACIN] = {
	8000/25-1,		/*   0 to 5[V] dammy */
	8000/25-1,		/*  10 +/-5[V] dammy */
	8000/25-1,		/*  20 +/-5[V] dammy */
	8000/25-1,		/*  30 +/-5[V] dammy */
	8000/25-1,		/*  40 +/-5[V] dammy */
	8000/25-1,		/*  50 +/-5[V] dammy */
	8000/25-1,		/*  60 +/-5[V] dammy */
	8000/25-1,		/*  70 +/-5[V] dammy */
	8000/25-1,		/*  80 +/-5[V] dammy */
	7600/25-1,		/*  90 +/-5[V] */
	5700/25-1,		/* 100 +/-5[V] */
	4800/25-1,		/* 110 +/-5[V] */
	4200/25-1,		/* 120 +/-5[V] */
	3600/25-1,		/* 130 +/-5[V] */
	3000/25-1,		/* 140 +/-5[V] */
	2700/25-1,		/* 150 +/-5[V] */
	2300/25-1,		/* 160 +/-5[V] */
	1900/25-1,		/* 170 +/-5[V] */
	1600/25-1,		/* 180 +/-5[V] */
	1400/25-1,		/* 190 +/-5[V] */
	1200/25-1,		/* 200 +/-5[V] */
	1000/25-1,		/* 210 +/-5[V] */
	 800/25-1,		/* 220 +/-5[V] */
	 600/25-1,		/* 230 +/-5[V] */
	 450/25-1,		/* 240 +/-5[V] */
	 450/25-1,		/* 250 +/-5[V] */
	 450/25-1,		/* 260 +/-5[V] */
	 425/25-1,		/* 270 +/-5[V] dammy */
	 425/25-1,		/* 280 +/-5[V] dammy */
	 425/25-1,		/* 290 +/-5[V] dammy */
	 425/25-1,		/* 300 +/-5[V] dammy */
	 425/25-1		/* 310 +/-5[V] dammy */
};

/*----- Displacement on-time of PFC outpu table for ignition -----*/
const unsigned short ushPfcIgnition[NUM_STEP_VACIN] = {
	6000/25,		/*   0 to 5[V] dammy */
	6000/25,		/*  10 +/-5[V] dammy */
	6000/25,		/*  20 +/-5[V] dammy */
	6000/25,		/*  30 +/-5[V] dammy */
	6000/25,		/*  40 +/-5[V] dammy */
	6000/25,		/*  50 +/-5[V] dammy */
	6000/25,		/*  60 +/-5[V] dammy */
	6000/25,		/*  70 +/-5[V] dammy */
	6000/25,		/*  80 +/-5[V] dammy */
	5400/25,		/*  90 +/-5[V] */
	4700/25,		/* 100 +/-5[V] */
	3800/25,		/* 110 +/-5[V] */
	3000/25,		/* 120 +/-5[V] */
	2600/25,		/* 130 +/-5[V] */
	2300/25,		/* 140 +/-5[V] */
	2000/25,		/* 150 +/-5[V] */
	1700/25,		/* 160 +/-5[V] */
	1600/25,		/* 170 +/-5[V] */
	1400/25,		/* 180 +/-5[V] */
	1200/25,		/* 190 +/-5[V] */
	1200/25,		/* 200 +/-5[V] */
	1100/25,		/* 210 +/-5[V] */
	1000/25,		/* 220 +/-5[V] */
	 800/25,		/* 230 +/-5[V] */
	 800/25,		/* 240 +/-5[V] */
	 700/25,		/* 250 +/-5[V] */
	 600/25,		/* 260 +/-5[V] */
	 500/25,		/* 270 +/-5[V] dammy */
	 400/25,		/* 280 +/-5[V] dammy */
	 300/25,		/* 290 +/-5[V] dammy */
	 200/25,		/* 300 +/-5[V] dammy */
	 100/25 		/* 310 +/-5[V] dammy */
};

/*----- Displacement on-time of PFC output table for inverter frequency sweep -----*/
const unsigned short ushPfcDim[NUM_STEP_VACIN] = {
	300/25,			/*   0 to 5[V] dammy */
	300/25,			/*  10 +/-5[V] dammy */
	300/25,			/*  20 +/-5[V] dammy */
	300/25,			/*  30 +/-5[V] dammy */
	300/25,			/*  40 +/-5[V] dammy */
	300/25,			/*  50 +/-5[V] dammy */
	300/25,			/*  60 +/-5[V] dammy */
	300/25,			/*  70 +/-5[V] dammy */
	300/25,			/*  80 +/-5[V] dammy */
	250/25,			/*  90 +/-5[V] */
	200/25,			/* 100 +/-5[V] */
	150/25,			/* 110 +/-5[V] */
	150/25,			/* 120 +/-5[V] */
	100/25,			/* 130 +/-5[V] */
	100/25,			/* 140 +/-5[V] */
	 75/25,			/* 150 +/-5[V] */
	 75/25,			/* 160 +/-5[V] */
	 75/25,			/* 170 +/-5[V] */
	 50/25,			/* 180 +/-5[V] */
	 50/25,			/* 190 +/-5[V] */
	 50/25,			/* 200 +/-5[V] */
	 50/25,			/* 210 +/-5[V] */
	 25/25,			/* 220 +/-5[V] */
	 25/25,			/* 230 +/-5[V] */
	 25/25,			/* 240 +/-5[V] */
	 25/25,			/* 250 +/-5[V] */
	 25/25,			/* 260 +/-5[V] */
	 25/25,			/* 270 +/-5[V] dammy */
	 25/25,			/* 280 +/-5[V] dammy */
	 25/25,			/* 290 +/-5[V] dammy */
	 25/25,			/* 300 +/-5[V] dammy */
	 25/25 			/* 310 +/-5[V] dammy */
};

/*************************************************************************
**
**  function name:
**  	fn_ini_ad_variables
**
**	function:
**		Initialize the all variables for A/D conversion.
**		This function is called only once at the top of main function.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_ini_ad_variables(void)
{
	unsigned char ucCnt;
	
	for (ucCnt = 0; ucCnt < NUM_TOTAL_ADCH; ucCnt++)
	{
		ucAdAve[ucCnt] = 0;			/* Clear the A/D average variables of the all channels */
	}
	ucVdcOutOld = 0;				/* Clear the A/D average variable of previous DC output voltage */
	ucVac = 0;						/* Clear the AC supply voltage value */
	fn_ini_status_variables();		/* Initialize the status variables */
}

/*************************************************************************
**	
**  function name:
**  	fn_ini_status_variables
**
**	function:
**		Initialize the lamp status variables for lamp off state.
**		This function is called at the top of main function and lamp off state.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_ini_status_variables(void)
{
	ucDimMode = DIM_OFF;		/* Set the dimming mode OFF */
	ucDimState = STATE_OFF;		/* Set the dimming state OFF */
	bVdcAutoControl = OFF;		/* Clear the VDC output auto control flag */
	bLampIgnition = OFF;		/* Clear the Lamp ignition flag */
	bDimmingReq = OFF;			/* Clear the Dimming request flag */
}

/*************************************************************************
**	
**  function name:
**  	fn_start_ad
**
**	function:
**		1. Clear the A/D sum buffer and addition counter of the all channels.
**		2. Clear the A/D channel pointer.
**		3. Start A/D conversion.
**		This function is called before forever loop and every 10ms interval process.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_start_ad(void)
{
	unsigned char ucCnt;
	
	for (ucCnt = 0; ucCnt < NUM_TOTAL_ADCH; ucCnt++)
	{
		ushAdSum[ucCnt] = 0;	/* Clear the A/D sum buffer of the all channels */
		ucAdCnt[ucCnt] = 0;		/* Clear the A/D addition counter of the all channels */
	}
	ucAdCh = 0;					/* Clear the A/D channel pointer */
	ADS = ucAni[ucAdCh];		/* Set the A/D channel */
	ADCS = 1;					/* Enable A/D conversion operation */
}

/*************************************************************************
**	
**  function name:
**  	fn_accumulate_ad
**
**	function:
**		1. Accumulate A/D value.
**		2. Count A/D accumulation.
**		3. Change AD conversion channel.
**		This function is called if the A/D conversion ends.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_accumulate_ad(void)
{
	ushAdSum[ucAdCh] += ADCRH;	/* Add the 8-bit A/D value for the later avaraging */
	ucAdCnt[ucAdCh]++;			/* Increase A/D addition counter */
	
	if (ucAdCnt[NUM_TOTAL_ADCH - 1] < 0xFF)	/* A/D addition counter does not overflow */
	{
		if (ucAdCh < NUM_TOTAL_ADCH - 1)	/* A/D channel does not over maximum channel */
		{
			ucAdCh++;			/* Increase A/D channel pointer */
		}
		else		/* A/D channel over maximum channel */
		{
			ucAdCh = 0;			/* Initialize A/D channel pointer */
		}
		ADS = ucAni[ucAdCh];	/* Update A/D conversion channel */
	}
	else			/* Fail-safe process to prevent overflow of A/D addition counter */
	{
		ADCS = 0;				/* Stop A/D conversion */
	}
	ADIF = 0;					/* Clear the INTAD interrupt request flag */
}

/*************************************************************************
**	
**  function name:
**  	fn_average_ad
**
**	function:
**		1. Stop A/D conversion.
**		2. Memorize previous DC output voltage.
**		3. Average A/d conversion value.
**		This function is called at 10ms interval process.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_average_ad(void)
{
	unsigned char ucCnt;
	
	ADCS = 0;			/* Stop A/D conversion */
	ADIF = 0;			/* Clear the INTAD interrupt request flag */
	
	ucVdcOutOld = ucAdAve[VDCOUT];		/* Memorize the A/D value of the previous DC output voltage */
	
	for (ucCnt = 0; ucCnt < NUM_TOTAL_ADCH; ucCnt++)	/* Averaging A/D value of all the channels */
	{
		if (ucAdCnt[ucCnt] > 0)			/* A/D value has added at least once */
		{
			ucAdAve[ucCnt] = (unsigned char)(ushAdSum[ucCnt] / ucAdCnt[ucCnt]);
			/* (8-bit average value) = (16-bit sum value) / (8-bit addition counter value) */
		}
		else							/* A/D value has never added */
		{
			ucAdAve[ucCnt] = 0;			/* Averaging value is 0 */
		}
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_check_request
**
**	function:
**		Change dimming mode by checking the value of volume input.
**		If the dimmong mode "ucDimMode" is changed, the "bDimmingReq" must
**		be changed too.
**		This function is called at 10ms interval process.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_check_request(void)
{
	switch (ucDimMode)		/* Check current dimming mode */
	{
		case DIM_OFF:
			if (ucAdAve[VOLUME] > AD_VOLUME_MAX)		/* Volume value > Max dimming level */
			{
				ucDimMode = DIM_IGNITION;	/* Set the dimming mode IGNITION */
				bDimmingReq = ON;			/* Set the dimming request flag */
			}
			break;
			
		case DIM_IGNITION:
			if (ucAdAve[VOLUME] < AD_VOLUME_OFF)		/* Volume value < Lamp off level */
			{
				ucDimMode = DIM_OFF;		/* Set the dimming mode OFF */
				bDimmingReq = ON;			/* Set the dimming request flag */
			}
			break;
			
		case DIM_MAX:
			if (ucAdAve[VOLUME] < AD_VOLUME_OFF)		/* Volume value < Lamp off level */
			{
				ucDimMode = DIM_OFF;		/* Set the dimming mode OFF */
				bDimmingReq = ON;			/* Set the dimming request flag */
			}
			else if (ucAdAve[VOLUME] < AD_VOLUME_MIN)	/* Volume value < Min dimming level */
			{
				ucDimMode = DIM_MIN;		/* Set the dimming mode MIN */
				bDimmingReq = ON;			/* Set the dimming request flag */
			}
			break;
			
		case DIM_MIN:
			if (ucAdAve[VOLUME] < AD_VOLUME_OFF)		/* Volume value < Lamp off level */
			{
				ucDimMode = DIM_OFF;		/* Set the dimming mode OFF */
				bDimmingReq = ON;			/* Set the dimming request flag */
			}
			else if (ucAdAve[VOLUME] > AD_VOLUME_MAX)	/* Volume value > Max dimming level */
			{
				ucDimMode = DIM_MAX;		/* Set the dimming mode MAX */
				bDimmingReq = ON;			/* Set the dimming request flag */
			}
			break;
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_detect_fault
**
**	function:
**		Emergency stop if no lamp or over voltage is detected.
**		This function is called at 10ms interval process.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_detect_fault(void)
{
	if (ucDimMode != DIM_OFF)		/* Under the dimming process */
	{
		if (ucAdAve[LAMP] > AD_LAMP_DETECT)		/* Lamp is not detected */
		{
			ucDimMode = DIM_OFF;		/* Set the dimming mode OFF */
			bDimmingReq = ON;			/* Set the dimming request flag */
		}
		
		if (ucAdAve[VDCOUT] > AD_VDCOUT_OVER)	/* Over voltage of DC output is detected */
		{
			ucDimMode = DIM_OFF;		/* Set the dimming mode OFF */
			bDimmingReq = ON;			/* Set the dimming request flag */
		}
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_control_vdc
**
**	function:
**		Auto control of DC output voltage by changing PFC output.
**		This function is called at 10ms interval process.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_control_vdc(void)
{
	if (bVdcAutoControl)		/* VDC output auto control flag is ON */
	{
		if (ucAdAve[VDCOUT] > AD_VDCOUT_MAX
		 && ucAdAve[VDCOUT] >= ucVdcOutOld
		 && TX0CR0 > PFC_ON_STEP)
		/*-------------------------------------------------------------------------
		 when the following conditions are all satisfied, decrease the PFC output ON time.
		1.sense voltage is over the upperlimit of the target value
		2.sense voltage increased or kept the same
		3.TX0CR0 register is still possible to be set to decrease the duty.
		-------------------------------------------------------------------------*/
		{
			TX0CR0 -= PFC_ON_STEP;			/* Decrease the PFC output ON time */
			TX0CR1 = PFC_CYCLE_RESTART;		/* Write the same value to TX0CR1 to apply PFC output ON time change */
		}
		
		else if (ucAdAve[VDCOUT] < AD_VDCOUT_MIN
		      && ucAdAve[VDCOUT] <= ucVdcOutOld
		      && TX0CR0 <= PFC_ON_MAX - PFC_ON_STEP)
		/*-------------------------------------------------------------------------
		 when the following conditions are all satisfied, increase the PFC output ON time.
		1.sense voltage is under the lowerlimit of the target value
		2.sense voltage decreased or kept the same
		3.TX0CR0 register is still possible to be set to increase the duty.
		-------------------------------------------------------------------------*/
		{
			TX0CR0 += PFC_ON_STEP;			/* Increase the PFC output ON time */
			TX0CR1 = PFC_CYCLE_RESTART;		/* Write the same value to TX0CR1 to apply PFC output ON time change */
		}
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_control_dim
**
**	function:
**		Dimming control according to the dimming request.
**		This function is called at 10ms interval process.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_control_dim(void)
{
	if (bDimmingReq)		/* Dimming request flag is ON */
	{
		switch (ucDimMode)		/* Check the Dimming mode */
		{
			case DIM_OFF:			/* lamp off mode */
				fn_dim_off();			/* lamp off process */
				break;
				
			case DIM_IGNITION:		/* lamp ignition mode */
				fn_dim_ignition();		/* lamp ignition process */
				break;
				
			case DIM_MIN:			/* darken to minimum level mode */
				fn_dim_min();			/* darken process to minimum level */
				break;
				
			case DIM_MAX:			/* brighten up to maximum level mode */
				fn_dim_max();			/* brighten up process to maximum level  */
				break;
				
			default:				/* Unknown request */
				fn_dim_off();			/* lamp off process */
				break;
		}
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_dim_off
**
**	function:
**		Lamp off process.
**		1. Stop the PFC output and inverter output.
**		2. Initialize the lamp status variables
**		3. Wait for 1s to prevent immediate reignition
**		This function is called if lamp off is requested.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_dim_off(void)
{
	unsigned char ucCnt;
	
	/* Stop the PFC output */
	PM3.1 = 1;			/* Set low level output using external pull-down register */
	TX0TMC = 0;			/* Stop the TMX0 operation for PFC output */
	CMP2EN = 0;			/* Stop the comparator operation for zero-current detection */
	
	/* Stop the inverter output */
	TX1TMC = 0;			/* Stop the TMX1 operation for inverter output */
	
	fn_ini_status_variables();	/* Initialize the lamp status variables */
	
	/* Wait for 1s to prevent immediate reignition */
	for (ucCnt = 0; ucCnt < 100; ucCnt++)		/* Count for 1s = 10ms * 100 */
	{
		while (!TMIFH1){}		/* Wait for TMH1 interrupt generated at 10ms intervals */
		TMIFH1 = 0;				/* Clear the TMH1 interrupt request flag */
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_dim_ignition
**
**	function:
**		Lamp ignition process.
**		1. Boost DC output voltage by PFC output.
**      2. Start preheating by inverter output.
**		3. Preheat for 400 ms.
**		4. Sweep the inverter frequency to ignite the lamp.
**		5. Waiting for 500 ms makes lamp stable.
**		This function is called if lamp ignition is requested.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_dim_ignition(void)
{
	switch (ucDimState)			/* Check the lamp dimming state */
	{
		case STATE_OFF:			/* lamp off state */
			
			ucVac = (unsigned char)(((unsigned short)(ucAdAve[VACIN] + 5) * 7) / 51);
			/*
				Calculate AC supply voltage
				
				y = ax + b
				
				y = ucVac : AC supply voltage step (ex. y = 9 -> AC supply voltage = 85-95 V)
				x = ucAdAve[VACIN] : A/D conversion result of AC supply voltage
				a = const. : 7 / 51
				b = const. : 5 * 7 / 51
			*/
			
			/* Start ignition process if AC supply voltage is in 85-265VAC */
			if (ucVac >= VACIN_MIN && ucVac <= VACIN_MAX)
			{
				CMP2EN = 1;						/* Enables comparator operation for the zero-current detection */
				
				TX0CR0 = ushPfcPreheat[ucVac];	/* Set the PFC output ON time under preheating */
				TX0CR1 = PFC_CYCLE_RESTART;		/* Set the cycle time to restart PFC output */
				TX0TMC = 1;						/* Enables TMX0 operation for the PFC control */
				PM3.1 = 0;						/* Set output mode to the PFC output port */
				
				ucDimCnt = 0;					/* Clear the Dimming counter */
				ucDimState = STATE_BOOST;		/* Set the DC output boost state */
			}
			
			/* Start ignition process if AC supply voltage is not in 85-265VAC */
			else
			{
				fn_ini_status_variables();				/* Initialize the lamp status variables */
			}
			break;
			
		case STATE_BOOST:			/* DC output boost state */
			
			/* Start inverter output when the DC output voltage reaches the target */
			if (ucAdAve[VDCOUT] > AD_VDCOUT_HBSTART)
			{
				TX1CR1 = INV_DUTY1_PREHEAT;			/* Set TX1CR1 for the high-side output */
				TX1CR2 = INV_DUTY2_PREHEAT;			/* Set TX1CR2 for the dead time and the low-side output */
				ushInvCycle = INV_CYCLE_PREHEAT;	/* Set variable for TX1CR3 rewriting */
				TX1CR3 = INV_CYCLE_PREHEAT;			/* Set TX1CR3 for the starting inverter cycle time */
				TX1TMC = 1;							/* Start TMX1 operation for inverter output */
				
				bVdcAutoControl = ON;				/* Set the VDC output auto control flag */
				ucDimCnt = 0;						/* Clear dimming time counter */
				ucDimState = STATE_PREHEAT;			/* Set the preheating state */
			}
			
			/* Measure the boost time until the DC output voltage reaches the target */
			else
			{
				ucDimCnt++;						/* Increment the dimming time counter */
				
				/* Lamp off when the boost time reaches the time limit */
				if (ucDimCnt >= COUNT_BOOST_FAIL)
				{
					fn_dim_off();				/* lamp off process */
				}
			}
			break;
			
		case STATE_PREHEAT:			/* Lamp preheating state */
			
			ucDimCnt++;							/* Increment the dimming time counter */
			
			/* End preheating when the target time has come */
			if (ucDimCnt >= COUNT_DIM_PREHEAT)
			{
				ucDimState = STATE_SWEEP;		/* Set the inverter frequency sweep state */
			}
			break;
			
		case STATE_SWEEP:			/* inverter frequency sweep state for ignition */
			
			TX1CR1 += INV_DUTY_SWEEP;			/* Set TX1CR1 for the high-side output */
			TX1CR2 += INV_DUTY_SWEEP;			/* Set TX1CR2 for the dead time and the low-side output */
			ushInvCycle += INV_CYCLE_SWEEP;		/* Increase the cycle time for the inverter frequency sweep */
			TMIFX1 = 0;							/* Clear the INTTMX1 interrupt request flag */
			TMMKX1 = 0;							/* Unmusk the INTTMX1 interrupt to update new inverter output */
			
			/* lamp ignition flag is OFF and DC output voltage is lower than the target voltage */
			if (!bLampIgnition && ucAdAve[0] < AD_VDCOUT_IGNITION)	/* Lamp ignition has happened */
			{
				TX0CR0 += ushPfcIgnition[ucVac];	/* Change the PFC output ON time corresponding to the load */
				TX0CR1 = PFC_CYCLE_RESTART;			/* Write the same value to TX0CR0 to update new PFC output */
				bLampIgnition = ON;					/* Set the Lamp ignition flag */
			}
			
			/* End sweep process when the frequency become the target value */
			if (ushInvCycle >= INV_CYCLE_MAX)
			{
				if (bLampIgnition)			/* The Lamp ignition has happened */
				{
					ucDimCnt = 0;					/* Clear the dimming time counter */
					ucDimState = STATE_LIGHTING;	/* Set the light up state */
				}
				else						/* The Lamp ignition has not happened */
				{
					fn_dim_off();					/* lamp off process */
				}
			}
			break;
			
		case STATE_LIGHTING:		/* Light up state for waiting for lamp stable state */
			
			ucDimCnt++;							/* Increment the dimming time counter */
			
			/* End waiting process for lamp stable when the target time has come */
			if (ucDimCnt >= COUNT_DIM_STABLE)
			{
				ucDimState = STATE_STABLE;		/* Set the lamp stable state */
				ucDimMode = DIM_MAX;			/* Set the dimming mode MAX */
				bDimmingReq = OFF;				/* Clear the dimming request flag */
			}
			break;
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_dim_min
**
**	function:
**		Darken to minimum level process by increasing inverter frequency.
**		This function is called if minimum dimming is requested.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_dim_min(void)
{
	if (ushInvCycle > INV_CYCLE_MIN)	/* Continue sweep process until the frequency become the target value */
	{
		TX1CR1 -= INV_DUTY_SWEEP;			/* Set TX1CR1 for the high-side output */
		TX1CR2 -= INV_DUTY_SWEEP;			/* Set TX1CR2 for the dead time and the low-side output */
		ushInvCycle -= INV_CYCLE_SWEEP;		/* Decrease the cycle time for the inverter frequency sweep */
		TMIFX1 = 0;							/* Clear the INTTMX1 interrupt request flag */
		TMMKX1 = 0;							/* Unmusk the INTTMX1 interrupt to update new inverter output */
		
		TX0CR0 -= ushPfcDim[ucVac];			/* Change the PFC output ON time corresponding to the load change */
		TX0CR1 = PFC_CYCLE_RESTART;			/* Write the same value to TX0CR0 to update new PFC output */
	}
	else								/* End sweep process when the frequency become the target value */
	{
		bDimmingReq = OFF;					/* Clear the Dimming request flag */
	}
}

/*************************************************************************
**	
**  function name:
**  	fn_dim_max
**
**	function:
**		Brighten up to maximum level process by decreasing inverter frequency.
**		This function is called if maximum dimming is requested.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void fn_dim_max(void)
{
	if (ushInvCycle < INV_CYCLE_MAX)	/* Continue sweep process until the frequency become the target value */
	{
		TX1CR1 += INV_DUTY_SWEEP;			/* Set TX1CR1 for the high-side output */
		TX1CR2 += INV_DUTY_SWEEP;			/* Set TX1CR2 for the dead time and the low-side output */
		ushInvCycle += INV_CYCLE_SWEEP;		/* Increase the cycle time for the inverter frequency sweep */
		TMIFX1 = 0;							/* Clear the INTTMX1 interrupt request flag */
		TMMKX1 = 0;							/* Unmusk the INTTMX1 interrupt to update new inverter output */
		
		TX0CR0 += ushPfcDim[ucVac];			/* Change the PFC output ON time corresponding to the load change */
		TX0CR1 = PFC_CYCLE_RESTART;			/* Write the same value to TX0CR0 to update new PFC output */
	}
	else								/* End sweep process when the frequency become the target value */
	{
		bDimmingReq = OFF;					/* Clear the Dimming request flag */
	}
}
